#!/usr/bin/ruby
# pdfpen_googledocs.rb
# Copyright 2010 by SmileOnMyMac, LLC dba Smile
# Version 1.0

# Prerequisite for accessing Google Spreadsheets
# gem sources -a http://gemcutter.org
# sudo gem install google-spreadsheet-ruby

require "rubygems"
require "google_spreadsheet"

google_username = "••FILL THIS IN•••"
google_password = "••FILL THIS IN•••"

class String

require "iconv"

# taken from: http://www.w3.org/International/questions/qa-forms-utf-8
   UTF8REGEX = /\A(?:                               # ?: non-capturing group (grouping with no back references)
                 [\x09\x0A\x0D\x20-\x7E]            # ASCII
               | [\xC2-\xDF][\x80-\xBF]             # non-overlong 2-byte
               |  \xE0[\xA0-\xBF][\x80-\xBF]        # excluding overlongs
               | [\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}  # straight 3-byte
               |  \xED[\x80-\x9F][\x80-\xBF]        # excluding surrogates
               |  \xF0[\x90-\xBF][\x80-\xBF]{2}     # planes 1-3
               | [\xF1-\xF3][\x80-\xBF]{3}          # planes 4-15
               |  \xF4[\x80-\x8F][\x80-\xBF]{2}     # plane 16
               )*\z/mnx

def utf8?
     self =~ UTF8REGEX
end
   
def utf16be_to_utf8
       ret = Iconv.iconv('UTF-8//IGNORE', 'UTF-16BE', (self[0,(self.length/2*2)] + "\000\000") ).first[0..-2]
       ret =~ /\x00\z/ ?  ret.sub!(/\x00\z/, '') : ret
       ret.utf8? ? ret : nil
end

def utf16le_to_utf8
       ret = Iconv.iconv('UTF-8//IGNORE', 'UTF-16LE', (self[0,(self.length/2*2)] + "\000\000") ).first[0..-2]
       ret =~ /\x00\z/ ?  ret.sub!(/\x00\z/, '') : ret
       ret.utf8? ? ret : nil
end

end

fields = Hash.new

# Parse fields input into a hash
$stdin.each do |line|
	items = line.split(/\t/);
	value = items[1].strip();
	if (value[0].to_int() == 0xFE && value[1].to_int() == 0xFF)
		value = value.utf16be_to_utf8()
	end
	if (value[0].to_int() == 0xFF && value[1].to_int() == 0xFE)
		value = value.utf16be_to_utf8()
	end
	fields[items[0]] = value
end

# Handle special fields
docTitle = fields['com.pdfpen.docTitle']
if (docTitle == nil || docTitle == "") then
	docTitle = "Form Responses"
end

docID = fields['com.pdfpen.docID']
if (docID == nil || docID == "") then
	docID = "Unknown ID"
end

# Start our Google Docs session
session = GoogleSpreadsheet.login(google_username, google_password)

sheet = nil

# search for a spreadsheet whose title contains the form's document ID
sheets = session.spreadsheets("title" => docID)
if (sheets.length == 0) then
	# create a new spreadsheet if we don't find an existing one
	sheet = session.create_spreadsheet(docTitle + " [" + docID + "]")
else
	sheet = sheets[0]
end

worksheet = sheet.worksheets[0];

topRowLastItem = 1
insertRow = 2

# If there are existing rows, pick the next row for insertion
if (worksheet.num_rows() > 0) then
	topRowLastItem = worksheet.num_cols() + 1
	insertRow = worksheet.num_rows() + 1
else
	# If there aren't any rows, start an empty one
	worksheet[1, 1] = ""
end

topRow = worksheet.rows[0]
debug = "insert row: " + insertRow.to_s() +  "\n"

fields.each_pair do |key, val|
	# make sure there's a header entry for each key in our form
	if (key[0,10] != "com.pdfpen") then
		insertCol = topRow.index(key)
		if (insertCol == nil) then
			worksheet[1, topRowLastItem] = key
			insertCol = topRowLastItem
			topRowLastItem = topRowLastItem + 1
			debug = debug + "Added key: " + key + "\n"
		else
			# correct for the fact that the [row,col] function for worksheets is 1-based
			insertCol = insertCol + 1
		end
		debug = debug + "worksheet[" + insertRow.to_s() + "," + insertCol.to_s() + "] = " + val + "\n"
		worksheet[insertRow, insertCol] = val
		debug = debug + key + " => " + val + "\n"
	end
end

# save our work
worksheet.save()
