#!/usr/bin/perl
# pdfpen_submit.cgi
# Copyright 2010 by SmileOnMyMac, LLC dba Smile
# Version 1.0

# Prerequisite for FDF parsing: PDF::FDF::Simple
# sudo perl -MCPAN -eshell
# cpan> install PDF::FDF::Simple

# Prerequisite for PDF parsing: CAM::PDF
# sudo perl -MCPAN -eshell
# cpan> install CAM::PDF

require CGI;
require XML::Simple;
require PDF::FDF::Simple;
require CAM::PDF;

$cgi = new CGI;

my $ok = 1;
my %fields = ();

my $debug_dump_dir = "••FILL THIS IN••";
my $cgi_bin_dir = "••FILL THIS IN OR SET TO UNDEF••";

$charset = "utf-8";
$contentType = $cgi->content_type();
if ($contentType =~ /([^;]+)\W*;\W*([^\n\r]+)/) {
	$contentType = $1;
	$charset = $2;
}

# Gather fields based on the content type
# Currently, the script supports:
#    XFDF
#    FDF
#    PDF
#    HTML forms

if ($contentType eq "application/vnd.adobe.xfdf") {
	$rawData = $cgi->param('POSTDATA');

	if ($debug_dump_dir) {
		open FILE, "> $debug_dump_dir/dump.xfdf" or $ok = 0;
		print FILE $cgi->content_type() . "\n";
		print FILE $rawData;
		close FILE;
	}

	$xml = new XML::Simple();
	$xfdf = $xml->XMLin($rawData,
		ForceArray => 1,
		KeyAttr    => {},
		);
	foreach my $field (@{$xfdf->{fields}[0]->{field}}) {
		$key = $field->{name};
		if ($key) {
			$value = $field->{value}[0];
	 		if (not ref($value)) {
				$fields{$key} = $value;
			}
		}
	}
	
	$originalID = $xfdf->{ids}[0]->{original};
	if ($originalID) {
		$fields{'com.pdfpen.docID'} = $originalID;
	}
}

# Need to do install PDF::FDF::Simple
if ($contentType eq "application/vnd.adobe.fdf" || $contentType eq "application/vnd.fdf") {
	$rawData = $cgi->param('POSTDATA');

	if ($debug_dump_dir) {
		open FILE, "> $debug_dump_dir/dump.fdf" or $ok = 0;
		print FILE $cgi->content_type() . "\n";
		print FILE $rawData;
		close FILE;
	}

	my $fdf = new PDF::FDF::Simple();
	$fdfcontent = $fdf->load($rawData);
	
	$originalIDRef = $fdf->attribute_id;
	if ($originalIDRef) {
		$originalID = @{$originalIDRef}[0];
		if ($originalID =~ m/<([^>]+)>/) {
			$originalID = $1;
		}
		$fields{'com.pdfpen.docID'} = $originalID;
	}	

	while ( my ($key, $value) = each(%$fdfcontent) ) {
		if ($value) {
			$fields{$key} = $value;
		}
	}
}

# Need to do install CAM::PDF
if ($contentType eq "application/pdf") {
	$rawData = $cgi->param('POSTDATA');
	$pdf = CAM::PDF->new($pdfData);
	
	foreach $key ($pdf->getFormFieldList()) {
		$value = $pdf->getFormField($key)->{value}->{value}->{V}->{value};
		if ($value) {
			$fields{$key} = $value;
		}
	}
}

if ($contentType eq "application/x-www-form-urlencoded" || $contentType eq "") {
	@names = $cgi->param;

	if ($debug_dump_dir) {
		open FILE, "> $debug_dump_dir/dump.html" or $ok = 0;
		print FILE $cgi->content_type() . "\n";
		foreach $key (@names) {
			$value = $cgi->param($key);
			print FILE $key . "\t" . $value . "\n";
		}
		close FILE;
	}

	foreach $key (@names) {
		$value = $cgi->param($key);
		if ($value) {
			$fields{$key} = $value;
		}
	}
}

my $lineCount = 0;

# pipe fields to Google Docs script
if ($debug_dump_dir) {
	open DEBUGFILE, "> $debug_dump_dir/outdata.dump"
}
open FILE, "| $cgi_bin_dir/pdfpen_googledocs.rb" or $ok = 0;
if ($ok) {
	while ( my ($key, $value) = each(%fields) ) {
		print FILE "$key\t$value\n";
		$lineCount++;
		if ($debug_dump_dir) {
			print DEBUGFILE "$key\t$value\n";
		}
	}
}
close FILE or $ok = 0;

if ($debug_dump_dir) {
	close DEBUGFILE;
}

if ($lineCount == 0) {
	$ok = 0;
}

# Send result
# The result must be a PDF to function in Adobe Reader.
# PDFpen and Acrobat can handle plain text and HTML.

print $cgi->header("application/pdf");
local *RESULTPDF;
if ($ok) {
	open RESULTPDF, "< $cgi_bin_dir/pdfpen_thankyou.pdf";
} else {
	open RESULTPDF, "< $cgi_bin_dir/pdfpen_sorry.pdf";
}
binmode(RESULTPDF);
while (read(RESULTPDF, $buffer, 102400)) { # 100K buffer
	print $buffer;
}

return 0;
